import json
import os

from circles_importer.importer import Importer
from data_source_local.src.data_source_enum import DataSource
from entity_type.entity_enum import EntityType
from language_local.lang_code import LangCode
from location_local.location_local_constants import LocationLocalConstants
from logger_local.Logger import Logger
from logger_local.LoggerComponentEnum import LoggerComponentEnum
from profile_local.comprehensive_profile import ComprehensiveProfilesLocal
from profile_local.profiles_local import ProfilesLocal
from user_context_remote.user_context import UserContext

from .constants import (PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
                        PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME)
from .search_reddit import Reddit

BRAND_NAME = os.environ.get('BRAND_NAME')
ENVIRONMENT_NAME = os.environ.get('ENVIRONMENT_NAME')
UNKNOWN_LOCATION = LocationLocalConstants.UNKNOWN_LOCATION_ID
object_to_insert = {
    'component_id': PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
    'component_name': PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME,
    'component_category': LoggerComponentEnum.ComponentCategory.Code.value,
    'developer_email': 'yoav.e@circ.zone'
}

logger = Logger.create_logger(object=object_to_insert)

if BRAND_NAME != 'Circlez':
    error = "please add BRAND_NAME to be Circlez to .env"
    logger.exception(error)
    raise Exception(error)

if ENVIRONMENT_NAME is None:
    error = "please add ENVIRONMENT_NAME to .env"
    logger.exception(error)
    raise Exception(error)


class RedditImporter(ProfilesLocal):
    def __init__(self):
        super().__init__()
        self.importer = Importer()
        self.user = UserContext()

    @staticmethod
    def process_reddit_user(reddit_user: dict) -> dict:

        comments = reddit_user['results']['profile']['comments']
        submissions = reddit_user['results']['profile']['submissions']

        comments_processed = []
        submissions_processed = []

        for comment in comments.new(limit=None):
            comments_processed.append(comment.body)

        for submission in submissions.new(limit=None):
            submissions_processed.append(submission.title)

        reddit_user['results']['profile']['name'] = reddit_user['results']['profile']['profile_name']  # profile-local expects name
        reddit_user['results']['profile']['comments'] = comments_processed
        reddit_user['results']['profile']['submissions'] = submissions_processed

        return reddit_user

    # TODO Should we have subreddit as parameter?
    def handle(self, event: dict = None) -> None:
        """
        purpose:
            collect data on reddit users for a certain subreddit
        args:
            event: None for manual input(terminal),
            or a json formatted as such:
            {
                    "subreddit_name": "subreddit_name",
                    "user_count": "user_count"
            }

        example:
            main({"subreddit_name": "python", "user_count": 10})
            saves 10 users that commented on the python subreddit
            to the database

            main(None) asks for user input for subreddit name and user count
        """

        logger.start(object={"event": event})

        # collect the subreddit name and user count
        reddit = Reddit()
        subreddit_name, user_count = reddit.get_subreddit_and_query(event)
        subreddit = reddit.reddit.subreddit(subreddit_name)

        ##########################################################################
        # Here we need to add the subreddit to the groups table in the database,
        # so we can later add the users to the users table with the group name
        # as a foreign key
        ##########################################################################

        reddit_users = reddit.get_reddit_users_by_subreddit(subreddit, user_count)

        ## add a new group with the name of the subreddit, if DNE

        # try to fetch the group by name from the DB
        # if DNE, create a new group with the name of the subreddit
        # and send a event to insert it, else, do nothing

        # TODO What about connecting the profiles to groups? - Without importing the groups there is no big value.
        # group_id = get_group_by_name(subreddit_name) // pseudo code
        if reddit_users:
            for reddit_user in reddit_users:
                ##this step is getting the data ready for the generic insert
                processed_user = self.process_reddit_user(reddit_user)["results"]

                # this step is inserting the user to the database
                # this needs to return the profile_id for us to be able to use the importer
                profile_id = ComprehensiveProfilesLocal.insert(profile_json=json.dumps(processed_user),
                                                               lang_code=LangCode.ENGLISH)
                if not profile_id:
                    logger.error("error inserting profile " + str(reddit_user))
                    continue

                ## insert to group profile the profile-id and the group-id
                # insert_to_group_profile(profile_id, group_id) // pseudo code

                ## register in importer
                self.importer.insert(
                    data_source_id=DataSource.REDDIT_DATA_SOURCE_ID.value,
                    location_id=UNKNOWN_LOCATION,
                    entity_type_id=EntityType.PERSONAL_PROFILE_ENTITY_TYPE_ID.value,
                    entity_id=profile_id,
                    url=f"https://www.reddit.com/r/{subreddit_name}",
                    user_id=self.user.get_effective_user_id()
                )

        else:
            logger.error("error no Reddit users found")

        logger.end()



from dotenv import load_dotenv

load_dotenv()
import os
import sys

sys.path.append(os.getcwd())
import praw
from logger_local.Logger import Logger
from logger_local.LoggerComponentEnum import LoggerComponentEnum

from profile_reddit_restapi_imp_python_package.constants import (
    PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
    PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME)
from profile_reddit_restapi_imp_python_package.search_reddit import Reddit

object_to_insert = {
    'component_id': PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
    'component_name': PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME,
    'component_category': LoggerComponentEnum.ComponentCategory.Code.value,
    'testing_framework': LoggerComponentEnum.testingFramework.pytest.value,
    'developer_email': 'yoav.e@circ.zone'
}

logger = Logger.create_logger(object=object_to_insert)

# Add the parent directory to the path so we can import the script
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

#


def test_authenticate_reddit():
    TEST_AUTHENTICATE_REDDIT_METHOD_NAME = "test_authenticate_reddit()"
    logger.start(TEST_AUTHENTICATE_REDDIT_METHOD_NAME)
    reddit = Reddit()._authenticate_reddit()

    result = isinstance(reddit, praw.Reddit)
    logger.end(TEST_AUTHENTICATE_REDDIT_METHOD_NAME, object={'result': result})
    assert result


def test_get_subreddit_and_query():
    TEST_GET_SUBREDDIT_AND_QUERY_METHOD_NAME = "test_get_subreddit_and_query()"
    FUNNY = "funny"
    USER_COUNT = 10
    logger.start(TEST_GET_SUBREDDIT_AND_QUERY_METHOD_NAME)
    request = {"subreddit_name": FUNNY, "user_count": USER_COUNT}
    subreddit_name, num = Reddit().get_subreddit_and_query(request=request)
    logger.info(f"subreddit_name {subreddit_name}")
    logger.info(f"num {num}")
    assert subreddit_name == "funny"
    assert num == 10
    logger.end(TEST_GET_SUBREDDIT_AND_QUERY_METHOD_NAME)



import os

import praw
import tqdm
from language_local.lang_code import LangCode
from logger_local.Logger import Logger
from logger_local.LoggerComponentEnum import LoggerComponentEnum

from .constants import (PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
                        PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME)

REDDIT_CLIENT_ID = os.environ.get('REDDIT_CLIENT_ID')
REDDIT_CLIENT_SECRET = os.environ.get('REDDIT_CLIENT_SECRET')
REDDIT_USERNAME = os.environ.get('REDDIT_USERNAME')

object_to_insert = {
    'component_id': PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
    'component_name': PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME,
    'component_category': LoggerComponentEnum.ComponentCategory.Code.value,
    'developer_email': 'yoav.e@circ.zone'
}

logger = Logger.create_logger(object=object_to_insert)


class Reddit:

    def __init__(self):
        logger.start("__init__")
        self.reddit = self._authenticate_reddit()
        logger.end("__init__")

    @staticmethod
    def _authenticate_reddit() -> praw.Reddit:
        logger.start()
        reddit = praw.Reddit(
            client_id=REDDIT_CLIENT_ID,
            client_secret=REDDIT_CLIENT_SECRET,
            user_agent=f"random_names (by u/{REDDIT_USERNAME})"
        )

        logger.end()
        return reddit

    @staticmethod
    def get_subreddit_and_query(request: dict = None, user_count: int = None, subreddit_name: str = None) -> tuple:
        logger.start()
        if request:
            subreddit_name, user_count = request['subreddit_name'], request['user_count']

        logger.end(object={'subreddit_name': subreddit_name, 'user_count': user_count})
        return subreddit_name, user_count

    @staticmethod
    def get_reddit_users_by_subreddit(subreddit, user_count: int):
        logger.start(object={'subreddit': subreddit.name, 'user_count': user_count})
        users = []
        total = user_count or float('inf')

        with tqdm.tqdm(total=total, desc="Getting users") as pbar:
            for submission in subreddit.new(limit=None):
                if len(users) >= total:
                    return users
                for comment in submission.comments.list():
                    logger.info("Reddit user comment")
                    if len(users) >= total:
                        return users
                    if comment.author.name == 'AutoModerator':
                        continue

                    users.append({
                        'results': {
                            'reaction': {
                                'value': comment.author.comment_karma,
                                'image': None,
                                'title': f'{comment.author.name} comment karma',
                                'description': None
                            },

                            'profile': {
                                'profile_name': comment.author.name,
                                'name_approved': True,
                                'lang_code': LangCode.ENGLISH.value,
                                'visibility_id': True,
                                'is_approved': True,
                                'profile_type_id': 1,
                                'stars': 2,
                                'last_dialog_workflow_state_id': 1,
                                'comments': comment.author.comments,
                                'submissions': comment.author.submissions,
                                'created_utc': comment.author.created_utc,
                                'has_verified_email': comment.author.has_verified_email,
                                'is_employee': comment.author.is_employee,
                                'is_mod': comment.author.is_mod,
                                'is_gold': comment.author.is_gold,
                                'link_karma': comment.author.link_karma
                            },

                            'storage': {
                                "url": comment.author.icon_img,
                                "filename": f'{comment.author.name}.jpg',
                                "file_type": "Profile Image"
                            }
                        }
                    }
                    )

                    pbar.update(1)

        logger.end(object={"len users": len(users)})  # This is huge
        return users



import setuptools

# Each Python project should have pyproject.toml or setup.py
# TODO: Please create pyproject.toml instead of setup.py (delete the setup.py)
# used by python -m build
# ```python -m build``` needs pyproject.toml or setup.py
# The need for setup.py is changing as of poetry 1.1.0 (including current pre-release) as we have moved away from needing to generate a setup.py file to enable editable installs - We might able to delete this file in the near future
setuptools.setup(
    name='profile-reddit-restapi-imp-local',
    version='0.0.22',  # https://pypi.org/project/profile-reddit-restapi-imp-local/
    author="Circles",
    author_email="info@circles.life",
    description="PyPI Package for Circles profile-local-reddit-restapi-imp Local/Remote Python",
    # TODO: Please update the long description and delete this line
    long_description="This is a package for sharing common XXX function used in different repositories",
    url="https://github.com/circles-zone/profile-local-reddit-restapi-imp-python-package",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "praw>=7.4.0",
        "tqdm>=4.64.1",

        "importer-local",
        "source-data-local",
        "entity-type-local",
        "language-local",
        "location-local",
        "logger-local",
        "profile-local",
        "user-context-remote"
    ]
)
